import sys


def is_enabled():
    try:
        from App.config import getConfiguration
        debug_mode = getConfiguration().debug_mode
    except:
        debug_mode = False
    if debug_mode:
        return True

    try:
        from zope import testrunner
        from zope.testrunner import options
    except ImportError:
        from zope.testing import testrunner
        from zope.testing.testrunner import options
    frame = sys._getframe(2)
    while frame is not None and frame.f_code is not testrunner.run.__code__:
        frame = frame.f_back
    if frame is not None:
        try:
            pm = options.get_options().post_mortem
            return bool(pm)
        except:
            # options not recognized by test runnergiven
            pass

    return False



try:
    import ipdb as pdb
except:
    import pdb

import transaction
from AccessControl import SecurityManagement

SecurityManagement.newSecurityManager(
    None, app.acl_users.getUser('admin'))

for path, obj in app.ZopeFind(app, search_sub=True,
                              obj_expr="id=='broken'"):
    try:
        split_path = path.rsplit('/', 1)
        if len(split_path) == 2:
            container_path, broken_id = split_path
            container = app.unrestrictedTraverse(container_path)
        else:
            container = app
            broken_id, = split_path
        if broken_id in container.objectIds():
            if container.manage_delObjects is not None:
                container.manage_delObjects([broken_id])
    except:
        import pdb
        import sys
        pdb.post_mortem(sys.exc_info()[2])
        raise

transaction.commit()



"""Monkey patches to various CMFPlone code that swallows errors we
might want to debug."""

from Products.CMFPlone.MigrationTool import MigrationTool

orig_upgrade = MigrationTool.upgrade


def upgrade(self, REQUEST=None, dry_run=None, swallow_errors=0):
    """Keep portal migrations from swallowing errors."""
    return orig_upgrade(self, REQUEST, dry_run, swallow_errors)



"""Hook the logger.error call."""

import sys
import re
import logging

import six


try:
    from ipdb import set_trace
    from ipdb import post_mortem
except:
    from pdb import set_trace
    from pdb import post_mortem

logger = logging.getLogger('Products.PDBDebugMode')

LoggerClass = logging.getLoggerClass()
orig_error = LoggerClass.error

ignore_matchers = (
    # There's a known error in ZCatalog where deleting a container
    # will generate these log errors for its children
    re.compile(
        '^uncatalogObject unsuccessfully attempted to uncatalog an '
        'object with a uid of ').search,
    # Ignore broken GenericSetup handlers
    re.compile(
        '^Step .* has an invalid import handler$').search,
    re.compile(
        '^Step .* has an invalid export handler$').search,
)


def error(self, msg, *args, **kw):
    """Drop into pdb when logging an error."""
    result = orig_error(self, msg, *args, **kw)

    if not isinstance(msg, six.string_types):
        msg = str(msg)

    for matcher in ignore_matchers:
        try:
            match = matcher(msg)
        except:
            logger.exception('Matcher %r failed for log message %r' %
                             (matcher, msg))
        else:
            if match:
                break
    else:
        if kw.get('exc_info'):
            type, value, traceback = sys.exc_info()
            post_mortem(traceback)
        else:
            set_trace()

    return result



"""Monkey patches to various ZCatalog code that swallows errors we
might want to debug."""

import sys
try:
    import ipdb as pdb
except:
    import pdb

from Products.ZCatalog.ZCatalog import ZCatalog

from six.moves import range


orig_catalog_object = ZCatalog.catalog_object


def catalog_object(self, obj, uid=None, idxs=None,
                   update_metadata=1, pghandler=None):
    """Wrap to do post_mortem debugging on error."""
    try:
        return orig_catalog_object(
            self, obj, uid=uid, idxs=idxs,
            update_metadata=update_metadata, pghandler=pghandler)
    except:
        t, v, tb = sys.exc_info()
        pdb.post_mortem(tb)
        raise


def refreshCatalog(self, clear=0, pghandler=None):
    """Don't swallow errors on object indexing errors."""

    cat = self._catalog
    paths = list(cat.paths.values())
    if clear:
        paths = tuple(paths)
        cat.clear()

    num_objects = len(paths)
    if pghandler:
        pghandler.init(
            'Refreshing catalog: %s' % self.absolute_url(1), num_objects)

    for i in range(num_objects):
        if pghandler:
            pghandler.report(i)

        p = paths[i]
        obj = self.resolve_path(p)
        if not obj:
            obj = self.resolve_url(p, self.REQUEST)
        if obj is not None:
            self.catalog_object(obj, p, pghandler=pghandler)

    if pghandler:
        pghandler.finish()



"""Monkey patches to various AT code that swallows errors we might
want to debug."""

from Products.CMFCore.utils import getToolByName


def initializeArchetype(self, **kwargs):
    """Don't swallow errors on AT content creation."""
    self.initializeLayers()
    self.markCreationFlag()
    self.setDefaults()
    if kwargs:
        kwargs['_initializing_'] = True
        self.edit(**kwargs)
    self._signature = self.Schema().signature()


def initializeATCT(self, **kwargs):
    """Don't swallow errors on ATCT content creation."""
    self.initializeLayers()
    self.markCreationFlag()
    self.setDefaults()
    if kwargs:
        self.edit(**kwargs)
    self._signature = self.Schema().signature()
    if self.isPrincipiaFolderish:
        self.copyLayoutFromParent()


def getCatalogsByType(self, portal_type):
    """Don't swallow catalog access errors."""
    catalogs = []
    catalog_map = getattr(self, 'catalog_map', None)
    if catalog_map is not None:
        names = self.catalog_map.get(portal_type, ['portal_catalog'])
    else:
        names = ['portal_catalog']
    for name in names:
        catalogs.append(getToolByName(self, name))
    return catalogs



from setuptools import setup, find_packages
import os

version = '1.4'

setup(name='Products.PDBDebugMode',
      version=version,
      description="Post-mortem debugging on Zope exceptions",
      long_description=open("README.txt").read() + "\n" + open(
          os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from
      # http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          "Environment :: Web Environment",
          "Topic :: Software Development :: Libraries :: Python Modules",
          "Framework :: Plone",
          "Framework :: Plone :: 5.2",
          "Framework :: Zope :: 4",
          "Operating System :: OS Independent",
          "Programming Language :: Python",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3.6",
          "Programming Language :: Python :: 3.7",
      ],
      keywords='',
      author='Ross Patterson',
      author_email='me@rpatterson.net',
      url='http://pypi.python.org/pypi/Products.PDBDebugMode',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'collective.monkeypatcher',
          # -*- Extra requirements: -*-
          'six',
      ],
      extras_require={
          'ipdb': ['ipdb>=0.3'],
          'zodb': ['zope.testrunner'],
          'zodb-testing': ['zope.testing'],
      },
      entry_points={
          # -*- Entry points: -*-
          'z3c.autoinclude.plugin': 'target = plone',
      },
      )



"""If debug-mode=on, Monkey-patch the zpublisher_exception_hook to
call pdb.post_mortem on an error and enable import of pdb (and ipdb) in
unprotected code.
"""

import sys
from types import ModuleType
import AccessControl

from Products.PDBDebugMode import debugger

try:
    from ZPublisher.Publish import publish
    ZOPE2 = True
except:
    try:
        from ZServer.ZPublisher.Publish import publish
        ZOPE2 = True
    except:
        ZOPE2 = False


if debugger.is_enabled():
    if ZOPE2:
        z2_enabled = ModuleType('z2_enabled')
        sys.modules['Products.PDBDebugMode.z2_enabled'] = z2_enabled
    else:
        z4_enabled = ModuleType('z4_enabled')
        sys.modules['Products.PDBDebugMode.z4_enabled'] = z4_enabled

    # Allow import of pdb in unprotected code
    AccessControl.allow_module('pdb')
    AccessControl.allow_module('ipdb')

    AccessControl.allow_module('Products.PDBDebugMode.debugger')
